'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

var LazyList = (function (Component$$1) {
  function LazyList(props) {
    Component$$1.call(this, props);
    this.scrollListener = this.scrollListener.bind(this);

    var windowHeight = props.windowHeight;
    var elementHeight = props.elementHeight;
    var children = props.children;
    var computedWindowHeight = windowHeight || document && document.body.clientHeight;
    var computedElementHeight = elementHeight;
    var showNum = Math.round(computedWindowHeight / computedElementHeight);
    var visibleChildren = children.slice(0, showNum);

    this.state = {
      showNum: showNum,
      visibleChildren: visibleChildren,
      scrolledNum: 0,
      topOffset: 0,
      bottomOffset: 0
    };
  }

  if ( Component$$1 ) LazyList.__proto__ = Component$$1;
  LazyList.prototype = Object.create( Component$$1 && Component$$1.prototype );
  LazyList.prototype.constructor = LazyList;

  LazyList.prototype.componentDidMount = function componentDidMount () {
    window.addEventListener('scroll', this.scrollListener);
  };

  LazyList.prototype.componentWillUnmount = function componentWillUnmount () {
    window.removeEventListener('scroll', this.scrollListener);
  };

  LazyList.prototype.componentWillReceiveProps = function componentWillReceiveProps (props) {
    var ref = this.state;
    var scrolledNum = ref.scrolledNum;
    var showNum = ref.showNum;
    var ref$1 = this.props;
    var children = ref$1.children;

    children.length !== props.children.length &&
      this.setState({ visibleChildren: props.children.slice(scrolledNum, scrolledNum + showNum + 1) });
  };

  LazyList.prototype.shouldComponentUpdate = function shouldComponentUpdate (props, state) {
    return this.state.topOffset !== state.topOffset ||
      this.props.children.length !== props.children.length
  };

  LazyList.prototype.scrollListener = function scrollListener () {
    var ref = this.state;
    var showNum = ref.showNum;
    var ref$1 = this.props;
    var children = ref$1.children;
    var elementHeight = ref$1.elementHeight;
    var topScrollOffset = ref$1.topScrollOffset;
    var onLoad = ref$1.onLoad;
    var windowHeight = ref$1.windowHeight;
    var bottomScrollOffset = ref$1.bottomScrollOffset;

    var fullHeight = children.length * elementHeight;
    var scrolled = (window && window.scrollY || 0) - topScrollOffset;
    var scrolledNum = Math.round((scrolled > 0 ? scrolled : 0) / elementHeight) - 1;

    var visibleChildren = children.slice(scrolledNum, scrolledNum + showNum + 1);
    var topOffset = scrolledNum * elementHeight;
    var bottomOffset = fullHeight - (topOffset + (showNum * elementHeight));
    var breakPoint = fullHeight - windowHeight - bottomScrollOffset;

    if (this.state.topOffset !== topOffset || this.state.bottomOffset !== bottomOffset) {
      this.setState({ topOffset: topOffset, bottomOffset: bottomOffset, scrolledNum: scrolledNum, visibleChildren: visibleChildren });
      scrolled >= breakPoint && onLoad();
    }
  };

  LazyList.prototype.render = function render () {
    var ref = this.state;
    var visibleChildren = ref.visibleChildren;
    var topOffset = ref.topOffset;
    var bottomOffset = ref.bottomOffset;
    return (
      React__default.createElement( 'div', null,
        React__default.createElement( 'div', { style: { marginTop: topOffset } }),
        React__default.createElement( 'div', { style: { position: 'relative', width: '100%' } }, visibleChildren),
        React__default.createElement( 'div', { style: { marginBottom: bottomOffset } })
      )
    )
  };

  return LazyList;
}(React.Component));

LazyList.propTypes = {
  children: React.PropTypes.array.isRequired,
  elementHeight: React.PropTypes.number.isRequired,
  windowHeight: React.PropTypes.number,
  topScrollOffset: React.PropTypes.number,
  bottomScrollOffset: React.PropTypes.number,
  onLoad: React.PropTypes.func
};

LazyList.defaultProps = {
  topScrollOffset: 0,
  bottomScrollOffset: 0
};

module.exports = LazyList;
